==> ewd-document-store-2.1.14.tgz: package/lib/proto/forEachLeafNode.js <==
/*

 ----------------------------------------------------------------------------
 | ewd-document-store: Persistent JavaScript Objects and Document Database  |
 |                      using Global Storage                                |
 |                                                                          |
 | Copyright (c) 2016-19 M/Gateway Developments Ltd,                        |
 | Redhill, Surrey UK.                                                      |
 | All rights reserved.                                                     |
 |                                                                          |
 | http://www.mgateway.com                                                  |
 | Email: rtweed@mgateway.com                                               |
 |                                                                          |
 |                                                                          |
 | Licensed under the Apache License, Version 2.0 (the "License");          |
 | you may not use this file except in compliance with the License.         |
 | You may obtain a copy of the License at                                  |
 |                                                                          |
 |     http://www.apache.org/licenses/LICENSE-2.0                           |
 |                                                                          |
 | Unless required by applicable law or agreed to in writing, software      |
 | distributed under the License is distributed on an "AS IS" BASIS,        |
 | WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
 | See the License for the specific language governing permissions and      |
 |  limitations under the License.                                          |
 ----------------------------------------------------------------------------

  18 October 2019

*/

const NON_EXISTENT = 0;

module.exports = function(callback) {
  // iterates through only leaf nodes with data values

  var isWithinRoot = function(signature, currentArray) {
    // has it moved beyond the root Global Node that originally seeded the loop?  If so, stop the loop

    var match = true;
    for (var i = 0; i < signature.length; i++) {
      if (signature[i].toString() !== currentArray[i].toString()) {
        match = false;
        break;
      }
    }
    return match;
  };

  var node = this._node;
  var ok = true;
  var quit;
  var documentStore = this.documentStore;
  var db = documentStore.db;
  var DocumentNode = documentStore.DocumentNode;
  var next_node = db.next_node;

  var subscripts = this.path;
  var data;
  if (db.dbx) {
    node.initialise = true;
    next_node = next_node.bind(this);
  }
  do {
    node = next_node(node);
    if (node.defined !== NON_EXISTENT) {
      if (isWithinRoot(subscripts, node.subscripts)) {
        data = node.data || '';
        if (callback) {
          quit = callback.call(this, data, new DocumentNode(node.global, node.subscripts));
          if (quit === true) ok = false;
        }
      }
      else {
        ok = false;
      }
    }
    else {
      ok = false;
    }
  } while (ok);
};

==> ewd-document-store-2.1.27.tgz: package/lib/proto/forEachLeafNode.js <==
/*

 ----------------------------------------------------------------------------
 | ewd-document-store: Persistent JavaScript Objects and Document Database  |
 |                      using Global Storage                                |
 |                                                                          |
 | Copyright (c) 2016-20 M/Gateway Developments Ltd,                        |
 | Redhill, Surrey UK.                                                      |
 | All rights reserved.                                                     |
 |                                                                          |
 | http://www.mgateway.com                                                  |
 | Email: rtweed@mgateway.com                                               |
 |                                                                          |
 |                                                                          |
 | Licensed under the Apache License, Version 2.0 (the "License");          |
 | you may not use this file except in compliance with the License.         |
 | You may obtain a copy of the License at                                  |
 |                                                                          |
 |     http://www.apache.org/licenses/LICENSE-2.0                           |
 |                                                                          |
 | Unless required by applicable law or agreed to in writing, software      |
 | distributed under the License is distributed on an "AS IS" BASIS,        |
 | WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
 | See the License for the specific language governing permissions and      |
 |  limitations under the License.                                          |
 ----------------------------------------------------------------------------

  29 January 2020

*/

const NON_EXISTENT = 0;

module.exports = function(callback) {
  // iterates through only leaf nodes with data values

  var isWithinRoot = function(signature, currentArray) {
    // has it moved beyond the root Global Node that originally seeded the loop?  If so, stop the loop

    var match = true;
    for (var i = 0; i < signature.length; i++) {
      if (signature[i].toString() !== currentArray[i].toString()) {
        match = false;
        break;
      }
    }
    return match;
  };

  var node = this._node;
  var ok = true;
  var quit;
  var documentStore = this.documentStore;
  var next_node = documentStore.db.next_node;

  var subscripts = this.path;
  var data;
  if (documentStore.db.dbx) {
    node.initialise = true;
    next_node = next_node.bind(this);
  }
  else {
    next_node = next_node.bind(this.documentStore.db);
  }
  do {
    node = next_node(node);
    if (node.defined !== NON_EXISTENT) {
      if (isWithinRoot(subscripts, node.subscripts)) {
        data = node.data || '';
        if (callback) {
          quit = callback.call(this, data, new documentStore.DocumentNode(node.global, node.subscripts));
          if (quit === true) ok = false;
        }
      }
      else {
        ok = false;
      }
    }
    else {
      ok = false;
    }
  } while (ok);
};
